/*------------------------------------------------------------------------------
-- This file is a part of the QLop Software
-- Copyright (C) 2015, Plasma Physics Laboratory - CNRS
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-------------------------------------------------------------------------------*/
/*-- Author : Alexis Jeandet
-- Mail : alexis.jeandet@member.fsf.org
----------------------------------------------------------------------------*/
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QThread>
#include "themisdatafile.h"
#include <QProgressDialog>
#include <QProgressBar>
#include <QListWidgetItem>
#include <QVBoxLayout>
#include <QWidget>
#include "folderview.h"
#include "filedownloader.h"

namespace Ui {
  class MainWindow;
}

class MainWindow : public QMainWindow
{
  Q_OBJECT

public:
  explicit MainWindow(int OMP_THREADS,QWidget *parent = 0);
  ~MainWindow();

public slots:
  void itemDoubleClicked(QListWidgetItem *item);
  void plotFile(const QString& File);
  void dataReady(QCPDataMap *ch1,QCPDataMap *ch2,QCPDataMap *ch3);
  void downloadData(const QDate &date);
  void updateProgress(int threadId,int percentProgress);
  void addFolderView();
  void fileDownloadComplete();
protected:
  void changeEvent(QEvent *e);

private:
  Ui::MainWindow *ui;
  QList<FolderView*> folderViews;
  QList<FileDownloader*> pendingDownloads;
  ThemisDataFile fileReader;
  QList<QProgressBar*> progress;
  QWidget* progressWidget;
  QVBoxLayout*progressLayout;
  int OMP_THREADS;
};

#endif // MAINWINDOW_H
